% Alarm bound for the pump throttle bush
close all
clear all
clc

% Constants
FForm = 5;              % Form factor based on the geometry of component
FMat = 0.002;           % Material factor of component
FKF = 0.8;              % Grain shape factor
FHs = 1;                % Grain hardness factor
F = FForm * FMat * FKF * FHs;

ps = 2660;              % sand density kg/m^3

csRef = 1;              % reference consentration
wRef = 10;              % Refereance speed

% Wear at failure
cTB = 0.19;         % [mm] TB clearance, failure at 3times this value
E_Fail = cTB*3;          

% Simulations
n = 100000;

E_sim = [];

% Variables
for q = 1:n                     % Run for each simulation
    % Random variables
    cs = normrnd(0.0139 , 0.002); % Solid content in oil
    csEq = cs*(1100/1150); % Equivalent solids consentration
    FKG = normrnd(0.04, 0.005); % Grain size factor. 
    wmix = normrnd(41.1, 3);    % Fluid velocity
    E_monthly = ((csEq/csRef)*(wmix/wRef).^3) * ((F*FKG)/(1+(cs/ps))) *...
        (30/365); % 30/365 per 30th day.
    E_sim(q) = E_monthly;
end

E_avg = mean(E_sim);

%----------------------------------------------------------------------
% Fit a distribution to the empirical data set
createFit(E_sim);       % Used app "Distribution Fitter" in Matlab
%---------------------------------------------------------------------- 

%----------------------------------------------------------------------
% Alarm bound at 5% 
max = 95;     % alarm at 5%
x = linspace(0, 0.000055);           % x-value for the empirical dist.
E_Limit = prctile(E_sim, max);       % 30 day degradation limit
% -----------------------------------------------------

E_Alarm = E_Fail - E_Limit;     % Alarm bound found

FD_TB = (E_Alarm/E_avg)*(30/365);
% Empirical PDF
Emp_dist = normpdf(x, ans.mu, ans.sigma);

% PLOT1 - EMpirical histogram of E_sim
figure(2);
histogram(E_sim,50,'Normalization','pdf');     %Empirical prob distribution
hold on;
xlim = get(gca,'xlim');                   %Get limits to use in theoretical
ylim = get(gca,'ylim');                   %Get limits to use in theoretical

plot(x, Emp_dist, 'r');
plot([E_Limit, E_Limit], ylim, 'g');

legend('Empirical','Empirical PDF','E_L_i_m_i_t')
title('pdf')
xlabel('E [mm]')
ylabel('pdf')
saveas(gca, 'AlarmB_TB.eps','epsc');
hold off
