% Simulations tool for comparing maintenance strategies
close all
clear all
clc

oilp = 610/1000;      % 70USD per barrel around 610NOK, 
Pr_Pump = 75000;      % barrels produces each day for a big subsea pump
n = 100;               % Number of simulations
ta = 5;               % Renewal interval AbM
rt = 30;              % Run-time, how long is the desired life-time

% Costs
c_intervention = 7500;                  % Intervention cost
cost_C = 40000 + c_intervention;        % CrM cost  + intervention
cost_I = 25000 + c_intervention;        % Ideal M cost + intervention
cost_A = 25000 + c_intervention;        % AbM cost + intervention
dt_cost = oilp * Pr_Pump;          % Down time cost pr day
%Downtime
max_dt = 30;    % maximum down time [days]
min_dt = 2;     % minimum down time [days]

% Ideal
sim_tot_cost_ideal = [];
sim_tot_years_ideal = [];
sim_tot_dt_ideal = [];
% Corrective
sim_tot_cost_CorM = [];
sim_tot_years_CorM = [];
sim_tot_dt_CorM = [];
% Age-based
sim_tot_cost_AbM = [];
sim_tot_years_AbM = [];
sim_tot_dt_AbM = [];
% CbM
sim_tot_cost_CbM = [];
sim_tot_years_CbM = [];
sim_tot_dt_CbM = [];

% Register what failure causes system failure
Failure_Errosion = 0;
Failure_Life = 0;
%Register all failure dates
FD_sim = [];

% MTTFs and failure rates for life-time modeled components
MTTF_Rotor = 508;
lambda_Rotor = 1/MTTF_Rotor;

MTTF_Stator = 128;
lambda_Stator = 1/MTTF_Stator;

MTTF_RadBear = 421;
lambda_RadBear = 1/MTTF_RadBear;

MTTF_Cooler = 256;
lambda_Cooler = 1/MTTF_Cooler;

MTTF_PowPen = 342;
lambda_PowPen = 1/MTTF_PowPen;

MTTF_Shaft = 249;
lambda_Shaft = 1/MTTF_Shaft;

MTTF_IntSeal = 109;
lambda_IntSeal = 1/MTTF_IntSeal;

MTTF_MechSeal = 54;
lambda_MechSeal = 1/MTTF_MechSeal;

MTTF_RadBear2 = 421;
lambda_RadBear2 = 1/MTTF_RadBear2;

MTTF_ThrustBear = 152;
lambda_ThrustBear = 1/MTTF_ThrustBear;

MTTF_PTT = 132;
MTTF_PTT_CCF = 365;
lambda_PTT = 1/MTTF_PTT;
lambda_PTT_CCF = 1/MTTF_PTT_CCF;

MTTF_Acc = 80.5;
MTTF_Acc_CCF = 228;
lambda_Acc = 1/MTTF_Acc;
lambda_Acc_CCF = 1/MTTF_Acc_CCF;

MTTF_FlowM = 170;
MTTF_FlowM_CCF = 480;
lambda_FlowM = 1/MTTF_FlowM;
lambda_FlowM_CCF = 1/MTTF_FlowM_CCF;

MTTF_Filter = 4660;
lambda_Filter = 1/MTTF_Filter;

MTTF_PVR = 55;
lambda_PVR = 1/MTTF_PVR;

MTTF3Way = 290;
lambda3Way = 1/MTTF3Way;

MTTF_AccPack = 2693;
lambda_AccPack = 1/MTTF_AccPack;

MTTF_MainSeal = 265;
lambda_MainSeal = 1/MTTF_MainSeal;

MTTF_FlangeSeal = 265;
lambda_FlangeSeal = 1/MTTF_FlangeSeal;

% Parameters for hydro-abrasive wear
ps = 2660;              % sand density kg/m^3
csRef = 1;              % reference consentration
wRef = 10;              % Refereance speed
FKF = 0.8;              % Grain shape factor
FHs = 1;                % Grain hardness factor
cs_mean = 0.0139;
cs_dev = 0.002; 
FKG_mean = 0.04;
FKG_dev = 0.005;

%Throttle bush
FForm_TB = 5; % Form factor of the geometry of the component
FMat_TB = 0.002;           % Material factor of component
F_TB = FForm_TB * FMat_TB * FKF * FHs;
wmix_mean_TB = 41.1;
wmix_dev_TB = 3;
E_Alarm_TB = 0.5700;

%Impeller inlet
FForm_Imp_Inl = 20; % Form factor of the geometry of the component
FMat_Imp_Inl = 2.3385;          % Material factor of component
F_Imp_Inl = FForm_Imp_Inl * FMat_Imp_Inl * FKF * FHs;
wmix_mean_Imp_Inl = 23.2;
wmix_dev_Imp_Inl = 2;
E_Alarm_Imp_Inl = 19.9589;      % Alarm bound found by MCS


% Impeller blades
FForm_Imp_Blade = 6; % Form factor of the geometry of the component eroded
FMat_Imp_Blade = 2.3385;           % Material factor of component
F_Imp_Blade = FForm_Imp_Blade * FMat_Imp_Blade * FKF * FHs;
E_Alarm_Imp_Blade = 3.5900;      % Alarm bound found by MCS
wmix_mean_Imp_Blade = 23.2;
wmix_dev_Imp_Blade = 2;


% Impeller outlet
FForm_Imp_Outl = 20; % Form factor based on the geometry 
%of the component 
FMat_Imp_Outl = 2.3385;          % Material factor of component
F_Imp_Outl = FForm_Imp_Outl * FMat_Imp_Outl * FKF * FHs;
wmix_mean_Imp_Outl = 25.2;
wmix_dev_Imp_Outl = 2;
E_Alarm_Imp_Outl = 3.9486;      % Alarm bound found by MCS


for k = 1:n         % the simulation loop
    % Ideal
    tot_cost_ideal = 0;           
    tot_dt_ideal = 0;
    % Corrective
    tot_cost_CorM = 0;           
    tot_dt_CorM = 0;
    % Age-based
    tot_cost_AbM = 0;           
    tot_dt_AbM = 0;
    % CbM
    tot_cost_CbM = 0;           
    tot_dt_CbM = 0;
    
    years = 0;
    counter = 1;            % determine row, next renewal phase (FD_sim)
    while years < rt
% Life-time exponential (No degradation)    
    %Motor module
        %component 1 - Rotor
        T_Rotor = rand;
        FD_Rotor = (log(1-T_Rotor))/(-lambda_Rotor);  % Fialure date based on T

        %component 2 - Stator
        T_Stator = rand;
        FD_Stator = (log(1-T_Stator))/(-lambda_Stator);  % Fialure date based on T
        
        %component 3 - Radial bearing
        T_RadBear = rand;
        FD_RadBear = (log(1-T_RadBear))/(-lambda_RadBear);  % Fialure date based on T
        
        %component 4 - Cooler
        T_Cooler = rand;
        FD_Cooler = (log(1-T_Cooler))/(-lambda_Cooler);  % Fialure date based on T
        
        %component 5 - Power Penetrator
        T_PowPen = rand;
        FD_PowPen = (log(1-T_PowPen))/(-lambda_PowPen);  % Fialure date based on T
        
    % Pump module
        
        %component 6 - Shaft
        T_Shaft = rand;
        FD_Shaft = (log(1-T_Shaft))/(-lambda_Shaft);  % Fialure date based on T
        
        %component 7 - Internal seals
        T_IntSeal = rand;
        FD_IntSeal = (log(1-T_IntSeal))/(-lambda_IntSeal);  % Fialure date based on T
        
        %component 8 - Mechanical seal
        T_MeachSeal = rand;
        FD_MeachSeal = (log(1-T_MeachSeal))/(-lambda_MechSeal);  % Fialure date based on T
        
        %component 9 - Radial Bearings
        T_RadBear2 = rand;
        FD_RadBear2 = (log(1-T_RadBear2))/(-lambda_RadBear2);  % Fialure date based on T
        
        %component 10 - Thrust bearing
        T_ThrustBear = rand;
        FD_ThrustBear = (log(1-T_ThrustBear))/(-lambda_ThrustBear);  % Fialure date based on T
        
        %component 12 - PTT1 and 2 (Run simuntaneously, so fail when the
        %last one fails) or because of CCF
        T_PTT_1 = rand;
        T_PTT_2 = rand;
        T_PTT_CCF = rand;
        FD_PTT_1 = (log(1-T_PTT_1))/(-lambda_PTT);  % Fialure date based on T
        FD_PTT_2 = (log(1-T_PTT_2))/(-lambda_PTT);  % Fialure date based on T
        FD_PTT_CCF = (log(1-T_PTT_CCF))/(-lambda_PTT_CCF);
        FD_PTT = min([FD_PTT_CCF max([FD_PTT_1 FD_PTT_2])]); % check what
        %failures come first of CCF or one of the other components
        
        %component 13 - accelerometer1 and 2 (Run simuntaneously, so fail 
        %when the last one fails) or when they fail due to a CCF
        T_Acc_1 = rand;
        T_Acc_2 = rand;
        T_Acc_CCF = rand;
        FD_Acc_1 = (log(1-T_Acc_1))/(-lambda_Acc);  % Fialure date based on T
        FD_Acc_2 = (log(1-T_Acc_2))/(-lambda_Acc);  % Fialure date based on T
        FD_Acc_CCF = (log(1-T_Acc_CCF))/(-lambda_Acc_CCF);
        FD_Acc = min([FD_Acc_CCF max([FD_Acc_1 FD_Acc_2])]); % check what failures 
        %come first of CCF or one of the other components
        
        %component 14 - Flow meter
        T_FlowM_1 = rand;
        T_FlowM_2 = rand;
        T_FlowM_CCF = rand;
        FD_FlowM_1 = (log(1-T_FlowM_1))/(-lambda_FlowM);  % Fialure date based on T
        FD_FlowM_2 = (log(1-T_FlowM_2))/(-lambda_FlowM);  % Fialure date based on T
        FD_FlowM_CCF = (log(1-T_FlowM_CCF))/(-lambda_FlowM_CCF); % ---||---
        FD_FlowM = min([FD_FlowM_CCF max([FD_FlowM_1 FD_FlowM_2])]);
        
    % Barrier fluid system    
        %component 15 - Filter
        T_Filter = rand;
        FD_Filter = (log(1-T_Filter))/(-lambda_Filter);  % Fialure date based on T
        
        %component PVR Paralell (FD = FD1+FD2, as one of them are on
        %standby until the first one fails)
        T_PVR_1 = rand;
        T_PVR_2 = rand;
        FD_PVR_1 = (log(1-T_PVR_1))/(-lambda_PVR);  % Fialure date based on T
        FD_PVR_2 = (log(1-T_PVR_2))/(-lambda_PVR);  % Fialure date based on T
        FD_PVR = FD_PVR_1 + FD_PVR_2;    % Look at when the paralell strcture 
        %fail, where one component is in stand-by 
        
        %component 17 - 3-way valve
        T3Way = rand;
        FD3Way = (log(1-T3Way))/(-lambda3Way);  % Fialure date based on T
        
        %component 18 - Accumulator pack
        T_AccPack = rand;
        FD_AccPack = (log(1-T_AccPack))/(-lambda_AccPack);  % Fialure date based on T
        
        %component 19 - Main seal
        T_MainSeal = rand;
        FD_MainSeal = (log(1-T_MainSeal))/(-lambda_MainSeal);  % Fialure date based on T
        
        %component 20 - Flange seals
        T_FlangeSeal = rand;
        FD_FlangeSeal = (log(1-T_FlangeSeal))/(-lambda_FlangeSeal);  % Fialure date based on T
        
        %Find failure date of life-time system
        FD_Life = min([FD_Rotor FD_Stator FD_RadBear FD_Cooler FD_PowPen...
            FD_Shaft FD_IntSeal FD_MeachSeal FD_RadBear2 FD_ThrustBear...
            FD_PTT FD_Acc FD_FlowM FD_Filter FD_PVR FD3Way FD_AccPack...
            FD_MainSeal FD_FlangeSeal]); 

    % Degradated components
        % Erossion Throttle bush
        E_tot_TB = 0;
        year_TB = 0;
        while E_tot_TB < E_Alarm_TB                    % Run every year
            cs_TB = normrnd(cs_mean , cs_dev); % Solid content in oil
            csEq_TB = (cs_TB)*(1100/1150); % Equiv. solids consentration
            FKG_TB = normrnd(FKG_mean, FKG_dev); % Grain size factor. 
            wmix_TB = normrnd(wmix_mean_TB, wmix_dev_TB);  % Fluid velocity
            E_TB = ((csEq_TB/csRef)*(wmix_TB/wRef).^3) *...
                ((F_TB*FKG_TB)/(1+(cs_TB/ps))); 
            E_tot_TB = E_tot_TB + E_TB;
            year_TB = year_TB + 1;
        end
        % Failure date of the Throttle bush / Alarm bound being reached
        FD_TB = year_TB - 1 + (E_tot_TB - E_Alarm_TB);
        
        % Erossion Impeller Inlet
        E_tot_Imp_Inl = 0;              % Erosion total when alarmbound
        %is reached
        year_Imp_Inl = 0;              % Year of alarm bound being reached
        while E_tot_Imp_Inl < E_Alarm_Imp_Inl    
            cs_Imp_Inl = normrnd(cs_mean , cs_dev); % Solid content in oil
            csEq_Imp_Inl = (cs_Imp_Inl)*(1100/1150); % Equivalent solids 
            %consentration
            FKG_Imp_Inl = normrnd(FKG_mean, FKG_dev); % Grain size factor
            wmix_Imp_Inl = normrnd(wmix_mean_Imp_Inl, wmix_dev_Imp_Inl); 
            % Fluid velocity
            E_Imp_Inl = ((csEq_Imp_Inl/csRef)*(wmix_Imp_Inl/...
                wRef).^3) * ((F_Imp_Inl*FKG_Imp_Inl)/...
                (1+(cs_Imp_Inl/ps))); 
            E_tot_Imp_Inl = E_tot_Imp_Inl + E_Imp_Inl;
            year_Imp_Inl = year_Imp_Inl + 1;
        end
        FD_Imp_Inl = year_Imp_Inl - 1 + (E_tot_Imp_Inl - E_Alarm_Imp_Inl); 
        
        % Erossion Impeller Blade
        E_tot_Imp_Blade = 0;   % Errosion total when alarmbound is reached
        year_Imp_Blade = 0;           % YEar of alarm bound being reached

        while E_tot_Imp_Blade < E_Alarm_Imp_Blade                  
            cs_Imp_Blade = normrnd(cs_mean , cs_dev); %Solid content in oil
            csEq_Imp_Blade = (cs_Imp_Blade)*(1100/1150); % Equivalent 
            %solids consentration
            FKG_Imp_Blade = normrnd(FKG_mean, FKG_dev); % Grain size factor 
            wmix_Imp_Blade = normrnd(wmix_mean_Imp_Blade,...
                wmix_dev_Imp_Blade);    % Fluid velocity
            E_Imp_Blade = ((csEq_Imp_Blade/csRef)*...
                (wmix_Imp_Blade/wRef).^3) *...
                ((F_Imp_Blade*FKG_Imp_Blade)/(1+(cs_Imp_Blade/ps))); 
            E_tot_Imp_Blade = E_tot_Imp_Blade + E_Imp_Blade;
            year_Imp_Blade = year_Imp_Blade + 1;
        end
        FD_Imp_Blade = year_Imp_Blade - 1 + (E_tot_Imp_Blade -...
            E_Alarm_Imp_Blade);  % Date of alarm bound being reached
        
        % Erossion Impeller Outlet
        E_tot_Imp_Outl = 0;  % Errosion total when alarmbound is reached
        year_Imp_Outl = 0;        % YEar of alarm bound being reached

        while E_tot_Imp_Outl < E_Alarm_Imp_Outl 
            cs_Imp_Outl = normrnd(cs_mean , cs_dev); % Solid content in oil
            csEq_Imp_Outl = (cs_Imp_Outl)*(1100/1150); %Equivalent
            %solids consentration
            FKG_Imp_Outl = normrnd(FKG_mean, FKG_dev); % Grain size factor
            wmix_Imp_Outl = normrnd(wmix_mean_Imp_Outl, wmix_dev_Imp_Outl);    % Fluid velocity
            E_Imp_Outl = ((csEq_Imp_Outl/csRef)*...
                (wmix_Imp_Outl/wRef).^3) * ((F_Imp_Outl*...
                FKG_Imp_Outl)/(1+(cs_Imp_Outl/ps))); 
            E_tot_Imp_Outl = E_tot_Imp_Outl + E_Imp_Outl;
            year_Imp_Outl = year_Imp_Outl + 1;
        end
        FD_Imp_Outl = year_Imp_Outl - 1 + (E_tot_Imp_Outl -...
            E_Alarm_Imp_Outl);  % Date of alarm bound being reached
        
        % Find sand eroded components FD
        FD_Ero = min([FD_TB FD_Imp_Inl FD_Imp_Blade FD_Imp_Outl]);
        
        % Store number of times system fails due to errosion or 
        %life model component
        if FD_Ero < FD_Life         
            Failure_Errosion = Failure_Errosion + 1;
        else
            Failure_Life = Failure_Life + 1;
        end
        % Find system failure date
        FD = min([FD_Life FD_Ero]);
        FD_sim(k,counter) = FD;      % Put the FD into storer
        counter = counter + 1; % move to next row-space in the 
        %cycle (coloumn)
        
      % Maintenance strategy
        %Ideal 
        dt_ideal = min_dt;
        cost_ideal = cost_I + (dt_ideal * dt_cost); 
        tot_cost_ideal =  tot_cost_ideal + cost_ideal; 
        
        % Corrective
        cost_CorM = cost_C + (max_dt * dt_cost);
        tot_cost_CorM =  tot_cost_CorM + cost_CorM; 
        tot_dt_CorM = tot_dt_CorM + max_dt;
        
        % Age-based
        if FD < ta 
            %  Calculate cost based on failure day
            if ta-FD > max_dt/365  % Max number of days out of operations
               dt_AbM = max_dt;
               cost_AbM = cost_C + (dt_AbM * dt_cost);  
               % It failed and costs are same as for CorM
            else                        % If the FD is lower than 30 days
                %the maintenance is already under planning, and will 
                %be performed at the planned time
                dt_AbM = (ta-FD) * 365;
                cost_AbM = cost_C + (dt_AbM * dt_cost);
            end   
        else           % If no failure before the renewal occurs 
            FD_AbM = 5;                     % FD would be 5 years
            dt_AbM = 2;                     % dt would be 1 day, or 0
            cost_AbM = cost_A + (dt_AbM * dt_cost);      
            % Only cost would be that of a planned downtime
        end
        % Collect the cost of this life-time
        tot_cost_AbM =  tot_cost_AbM + cost_AbM; 
        tot_dt_AbM = tot_dt_AbM + dt_AbM;
        
        % Condition-based Maintenance with corrective
        if FD_Ero < FD_Life
            dt_CbM = min_dt;
            cost_CbM = cost_ideal; 
            tot_cost_CbM =  tot_cost_CbM + cost_CbM; 
        else 
            dt_CbM = max_dt;
            cost_CbM = cost_CorM;
            tot_cost_CbM =  tot_cost_CbM + cost_CbM; 
            tot_dt_CbM = tot_dt_CbM + max_dt;
        end
        
        years = years + FD;
    end
    % Ideal
    sim_tot_cost_ideal(k) = tot_cost_ideal;  % Collect the cost of 
    %all simulations life-time
    sim_tot_years_ideal(k) = years;
    sim_tot_dt_ideal(k) = tot_dt_ideal;
    
    % Corrective
    sim_tot_cost_CorM(k) = tot_cost_CorM;   % Collect the cost of all 
    %simulations life-time
    sim_tot_years_CorM(k) = years;
    sim_tot_dt_CorM(k) = tot_dt_CorM;
    
    % Age-based
    sim_tot_cost_AbM(k) = tot_cost_AbM;         % Collect the cost of all 
    %simulations life-time
    sim_tot_years_AbM(k) = years;
    sim_tot_dt_AbM(k) = tot_dt_AbM;
    
    % CbM
    sim_tot_cost_CbM(k) = tot_cost_CbM;         % Collect the cost of all 
    %simulations life-time
    sim_tot_years_CbM(k) = years;
    sim_tot_dt_CbM(k) = tot_dt_CbM;
end

% Averages of all simulations

% Ideal
avg_life_cost_ideal = mean(sim_tot_cost_ideal); % Calculate average cost 
%of all life-times
avg_life_ideal = mean(sim_tot_years_ideal);
avg_dt_ideal = mean(tot_dt_ideal);
avg_availability_ideal = 1 - (avg_dt_ideal/(365*avg_life_ideal));

% Corrective
avg_life_cost_CorM = mean(sim_tot_cost_CorM);     
avg_life_CorM = mean(sim_tot_years_CorM);
avg_dt_CorM = mean(tot_dt_CorM);
avg_availability_CorM = 1 - (avg_dt_CorM/(365*avg_life_CorM));

% Age-based
avg_life_cost_AbM = mean(sim_tot_cost_AbM);   
avg_life_AbM = mean(sim_tot_years_AbM);
avg_dt_AbM = mean(tot_dt_AbM);            
avg_availability_AbM = 1 - (avg_dt_AbM/(365*avg_life_AbM));

% CbM
avg_life_cost_CbM = mean(sim_tot_cost_CbM);    
avg_life_CbM = mean(sim_tot_years_CbM);
avg_dt_CbM = mean(tot_dt_CbM);             
avg_availability_CbM = 1 - (avg_dt_AbM/(365*avg_life_CbM));

% Line graph
figure(1); 
x1 = linspace(0,n,n);
plot(x1, sim_tot_cost_ideal, 'm','linewidth',1.5);
hold on;
plot(x1, sim_tot_cost_CorM, 'r','linewidth',1.5);
plot(x1, sim_tot_cost_AbM, 'g','linewidth',1.5);
plot(x1, sim_tot_cost_CbM, 'k','linewidth',1.5);
% Plot averages
xlim = get(gca,'xlim');                     %Get limits to use on averages
ylim = get(gca,'ylim');                     %Get limits to use on averages
plot(xlim, [avg_life_cost_ideal, avg_life_cost_ideal],'Color',...
    [193/256 60/256 108/256],'linewidth',1.5);
plot(xlim, [avg_life_cost_CorM,avg_life_cost_CorM],'Color',...
    [193/256 90/256 99/256],'linewidth',1.5);
plot(xlim, [avg_life_cost_AbM, avg_life_cost_AbM],'Color',...
    [70/256 148/256 73/256],'linewidth',1.5);
plot(xlim, [avg_life_cost_CbM, avg_life_cost_CbM],'Color',...
    [85/256 85/256 85/256],'linewidth',1.5);
% Name graphs
legend('Ideal maintenance cost', 'Corrective maintenance cost',...
    'Age-based maintenance cost', 'Condition-based maintenance cost',...
    'Avereage ideal maintnance cost',...
    'Avereage corrective... maintenance cost',...
    'Avereage age-based maintenance cost',...
    'Avereage condition-based maintenance cost');
title('Life-time simulation cost');
xlabel('Life-time, i');
ylabel('Cost [kNOK]');